@@ -175,17 +175,18 @@ arp_packet(struct interface *ifp, uint8_t *data, size_t len)
 	}
 }
 
-void
+static void
 arp_close(struct interface *ifp)
 {
 	struct iarp_state *state;
 
-	if ((state = ARP_STATE(ifp)) != NULL && state->bpf_fd != -1) {
-		eloop_event_delete(ifp->ctx->eloop, state->bpf_fd);
-		bpf_close(ifp, state->bpf_fd);
-		state->bpf_fd = -1;
-		state->bpf_flags |= BPF_EOF;
-	}
+	if ((state = ARP_STATE(ifp)) == NULL || state->bpf_fd == -1)
+		return;
+
+	eloop_event_delete(ifp->ctx->eloop, state->bpf_fd);
+	bpf_close(ifp, state->bpf_fd);
+	state->bpf_fd = -1;
+	state->bpf_flags |= BPF_EOF;
 }
 
 static void
