@@ -1,6 +1,6 @@
 /*
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2018 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2019 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -40,11 +40,13 @@
 #include "arp.h"
 
 struct ipv4ll_state {
+	struct in_addr pickedaddr;
 	struct ipv4_addr *addr;
 	struct arp_state *arp;
 	unsigned int conflicts;
 	struct timespec defend;
 	char randomstate[128];
+	bool seeded;
 	uint8_t down;
 };
 
@@ -66,16 +68,9 @@ void ipv4ll_handle_failure(void *);
 int ipv4ll_recvrt(int, const struct rt *);
 #endif
 
-#define	ipv4ll_free(ifp)		ipv4ll_freedrop((ifp), 0);
-#define	ipv4ll_drop(ifp)		ipv4ll_freedrop((ifp), 1);
-void ipv4ll_freedrop(struct interface *, int);
-#else
-#define	IPV4LL_STATE_RUNNING(ifp)	(0)
-#define	ipv4ll_subnetroute(route, ifp)	(0)
-#define	ipv4ll_defaultroute(route, ifp)	(0)
-#define	ipv4ll_handlert(a, b, c)	(0)
-#define	ipv4ll_free(a)			{}
-#define	ipv4ll_drop(a)			{}
+void ipv4ll_reset(struct interface *);
+void ipv4ll_drop(struct interface *);
+void ipv4ll_free(struct interface *);
 #endif
 
 #endif
