@@ -1,6 +1,6 @@
 /*
  * dhcpcd - route management
- * Copyright (c) 2006-2018 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2019 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -272,7 +272,7 @@ rt_kfree(struct rt *rt)
 /* If something other than dhcpcd removes a route,
  * we need to remove it from our internal table. */
 void
-rt_recvrt(int cmd, const struct rt *rt)
+rt_recvrt(int cmd, const struct rt *rt, pid_t pid)
 {
 	struct dhcpcd_ctx *ctx;
 	struct rt *f;
@@ -288,8 +288,11 @@ rt_recvrt(int cmd, const struct rt *rt)
 			rt_free(f);
 		}
 		if ((f = rt_find(&ctx->routes, rt)) != NULL) {
+			char buf[32];
+
 			TAILQ_REMOVE(&ctx->routes, f, rt_next);
-			rt_desc("deleted", f);
+			snprintf(buf, sizeof(buf), "pid %d deleted", pid);
+			rt_desc(buf, f);
 			rt_free(f);
 		}
 		break;
@@ -303,7 +306,7 @@ rt_recvrt(int cmd, const struct rt *rt)
 		break;
 	}
 
-#if defined(INET) && defined(HAVE_ROUTE_METRIC)
+#if defined(IPV4LL) && defined(HAVE_ROUTE_METRIC)
 	if (rt->rt_dest.sa_family == AF_INET)
 		ipv4ll_recvrt(cmd, rt);
 #endif
